@@ -1,14 +1,13 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2021 -- leonerd@leonerd.org.uk
 
-package Tangence::Meta::Argument;
+use v5.26;
+use Object::Pad 0.43;
 
-use strict;
-use warnings;
-
-our $VERSION = '0.25';
+package Tangence::Meta::Argument 0.28;
+class Tangence::Meta::Argument :strict(params);
 
 =head1 NAME
 
@@ -47,12 +46,8 @@ Type of the arugment as a L<Tangence::Meta::Type> reference
 
 =cut
 
-sub new
-{
-   my $class = shift;
-   my %args = @_;
-   bless \%args, $class;
-}
+has $name :reader :param = undef;
+has $type :reader :param;
 
 =head1 ACCESSORS
 
@@ -66,12 +61,6 @@ Returns the name of the class
 
 =cut
 
-sub name
-{
-   my $self = shift;
-   return $self->{name};
-}
-
 =head2 type
 
    $type = $argument->type
@@ -80,12 +69,6 @@ Return the type as a L<Tangence::Meta::Type> reference.
 
 =cut
 
-sub type
-{
-   my $self = shift;
-   return $self->{type};
-}
-
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
