@@ -1,14 +1,13 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2012-2017 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2012-2021 -- leonerd@leonerd.org.uk
 
-package Tangence::Meta::Field;
+use v5.26;
+use Object::Pad 0.41;
 
-use strict;
-use warnings;
-
-our $VERSION = '0.25';
+package Tangence::Meta::Field 0.28;
+class Tangence::Meta::Field :strict(params);
 
 =head1 NAME
 
@@ -46,12 +45,8 @@ Type of the field as a L<Tangence::Meta::Type> reference
 
 =cut
 
-sub new
-{
-   my $class = shift;
-   my %args = @_;
-   bless \%args, $class;
-}
+has $name :param :reader;
+has $type :param :reader;
 
 =head1 ACCESSORS
 
@@ -65,12 +60,6 @@ Returns the name of the field
 
 =cut
 
-sub name
-{
-   my $self = shift;
-   return $self->{name};
-}
-
 =head2 type
 
    $type = $field->type
@@ -79,12 +68,6 @@ Return the type as a L<Tangence::Meta::Type> reference.
 
 =cut
 
-sub type
-{
-   my $self = shift;
-   return $self->{type};
-}
-
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
